--- old/usr/src/uts/common/fs/zfs/spa.c	2014-10-16 19:15:50.998712281 +0200
+++ new/usr/src/uts/common/fs/zfs/spa.c	2014-10-16 19:15:51.004146238 +0200
@@ -1717,8 +1717,8 @@
 	case SPA_LOG_MISSING:
 		/* need to recheck in case slog has been restored */
 	case SPA_LOG_UNKNOWN:
-		rv = (dmu_objset_find(spa->spa_name, zil_check_log_chain,
-		    NULL, DS_FIND_CHILDREN) != 0);
+		rv = (dmu_objset_find_parallel(spa->spa_name,
+		    zil_check_log_chain, NULL, DS_FIND_CHILDREN) != 0);
 		if (rv)
 			spa_set_log_state(spa, SPA_LOG_MISSING);
 		break;
@@ -2660,7 +2660,7 @@
 
 		tx = dmu_tx_create_assigned(spa_get_dsl(spa),
 		    spa_first_txg(spa));
-		(void) dmu_objset_find(spa_name(spa),
+		(void) dmu_objset_find_parallel(spa_name(spa),
 		    zil_claim, tx, DS_FIND_CHILDREN);
 		dmu_tx_commit(tx);
 
@@ -2852,7 +2852,7 @@
  */
 static int
 spa_open_common(const char *pool, spa_t **spapp, void *tag, nvlist_t *nvpolicy,
-    nvlist_t **config)
+    nvlist_t **config, int lock)
 {
 	spa_t *spa;
 	spa_load_state_t state = SPA_LOAD_OPEN;
@@ -2867,7 +2867,7 @@
 	 * up calling spa_open() again.  The real fix is to figure out how to
 	 * avoid dsl_dir_open() calling this in the first place.
 	 */
-	if (mutex_owner(&spa_namespace_lock) != curthread) {
+	if (lock && (mutex_owner(&spa_namespace_lock) != curthread)) {
 		mutex_enter(&spa_namespace_lock);
 		locked = B_TRUE;
 	}
@@ -2964,13 +2964,19 @@
 spa_open_rewind(const char *name, spa_t **spapp, void *tag, nvlist_t *policy,
     nvlist_t **config)
 {
-	return (spa_open_common(name, spapp, tag, policy, config));
+	return (spa_open_common(name, spapp, tag, policy, config, 1));
 }
 
 int
 spa_open(const char *name, spa_t **spapp, void *tag)
 {
-	return (spa_open_common(name, spapp, tag, NULL, NULL));
+	return (spa_open_common(name, spapp, tag, NULL, NULL, 1));
+}
+
+int
+spa_open_lock(const char *name, spa_t **spapp, void *tag, int lock)
+{
+	return (spa_open_common(name, spapp, tag, NULL, NULL, lock));
 }
 
 /*
@@ -3154,7 +3160,7 @@
 	spa_t *spa;
 
 	*config = NULL;
-	error = spa_open_common(name, &spa, FTAG, NULL, config);
+	error = spa_open_common(name, &spa, FTAG, NULL, config, 1);
 
 	if (spa != NULL) {
 		/*
